import listen from 'test-listen';
import fetch from 'isomorphic-unfetch';
import _$http from 'http';
const {
  createServer
} = _$http;
import _$url from 'url';
const {
  parse: parseUrl
} = _$url;
import _$next_dist_next_server_server_api_utils_js from 'next/dist/next-server/server/api-utils.js';
const {
  apiResolver
} = _$next_dist_next_server_server_api_utils_js;

/**
 * Uses Next's internal `apiResolver` to execute api route handlers in a
 * Next-like testing environment.
 *
 * `requestPatcher/responsePatcher` are functions that receive an
 * IncomingMessage and ServerResponse object respectively. Use these functions
 * to edit the request and response before they're injected into the api
 * handler.
 *
 * `params` are passed directly to the api handler and represent processed
 * dynamic routes. This should not be confused with query string parsing, which
 * is handled automatically.
 *
 * `handler` is the actual api handler under test. It should be an async
 * function that accepts NextApiRequest and NextApiResult objects (in that
 * order) as its two parameters.
 *
 * `test` should be a function that returns a promise (or async) where test
 * assertions can be run. This function receives one parameter: fetch, which is
 * unfetch's `fetch(...)` function but with the first parameter omitted.
 */
export async function testApiHandler({
  requestPatcher,
  responsePatcher,
  params,
  handler,
  test
}) {
  let server = null;

  try {
    const localUrl = await listen(server = createServer((req, res) => {
      requestPatcher && requestPatcher(req);
      responsePatcher && responsePatcher(res);
      /**
       *? From next internals:
       ** apiResolver(
       **    req: IncomingMessage,
       **    res: ServerResponse,
       **    query: any,
       **    resolverModule: any,
       **    apiContext: __ApiPreviewProps,
       **    propagateError: boolean,
       **    onError?: ({ err }: { err: any }) => Promise<void>
       ** )
       */

      void apiResolver(req, res, { ...parseUrl(req.url || '', true).query,
        ...params
      }, handler, // eslint-disable-next-line @typescript-eslint/no-explicit-any
      undefined, true, // eslint-disable-next-line @typescript-eslint/no-explicit-any
      undefined);
    }));
    await test({
      fetch: init => fetch(localUrl, init)
    });
  } finally {
    var _server;

    (_server = server) === null || _server === void 0 ? void 0 : _server.close();
  }
}